//====================================================================================//
//                                                                                    //
//                                    Compile_DSC                                     //
//                                                                                    //
//====================================================================================//
//  This File:   compile_dsc.C               Language: C++  (xlC and CC)              //
//  Software:    PEPS 2006                   Advisor: Brigitte.Plateau@imag.fr        //
//====================================================================================//
//  Creation:    19/dec/95                        by: Paulo.Fernandes@imag.fr         //
//  Last Change: 13/feb/03                            by: Anne.Benoit@imag.fr         //
//====================================================================================//
#include "peps.h"

peps_name filename;

//====================================================================================//
void Compile_DSC(bool aggr_too, bool lump_too)                  //   Main Operation   //
//====================================================================================//
{                          //     SAN model name in all directories handled by PEPS
 peps_name model_name;     // current directory (just the name, no paths, no extensions)
 peps_name san_level_name; // directory where the textual network  is      (./*.san)
 peps_name des_level_name; // directory to store the textual descriptor    (./des/*.des)
 peps_name dsc_level_name; // directory to store the internal descriptor   (./dsc/*.dsc)
 peps_name agg_level_name; // directory to store the aggregated descriptor (./dsc/*.dsc)

 Timer     T;              // timer to inform total time spent
 aut_st    agreg_map;      // aggregation map to perform aggregation
 bool      lu_too;         // the descriptor must be LU decomposed (generates a .lud)

 cout << "\nCompilation of a SAN model (Internal Descriptor Generation)";
 T.clear();
 if (!strcmp(filename, ""))
   Ask_an_Existing_File_Name(san_level_name, san_file);
 else
   strcpy(san_level_name, filename);
 
 Get_Only_the_File_Name(san_level_name, model_name);
 strcpy(des_level_name, "des/");  strcat(des_level_name, model_name);
 strcpy(dsc_level_name, "dsc/");  strcat(dsc_level_name, model_name);
 strcpy(agg_level_name, "dsc/");
 cout << "\nCompile_Network\n";
 if (Permission_to_Proceed(dsc_level_name, dsc_file)) {            // .des compilation
   T.start();
   cout << "Compile_Function_Table\n";
   Compile_Function_Table        (des_level_name, dsc_level_name);
   cout << "Compile_Descriptor\n";
   Compile_Descriptor            (des_level_name, dsc_level_name);
   cout << "Compile_Reacheable_SS\n";
   Compile_Reacheable_SS         (des_level_name, dsc_level_name);
   cout << "Compile_Dictionary\n";
   Compile_Dictionary            (des_level_name, dsc_level_name);
   cout << "Compile_Integration_Function\n";
   Compile_Integration_Function  (des_level_name, dsc_level_name);
   T.stop();
   if (aggr_too || lump_too) {                                     // aggregation or lump
     cout << "Agregation of a SAN model\n"; 
     Ask_Agregation_Map(agreg_map);
     cout << "\nYou need a new file name to the aggregate model";
     Ask_a_File_Name(model_name, dsc_file);
     strcat(agg_level_name, model_name);
     if (Permission_to_Proceed(agg_level_name, dsc_file)) {
       T.start();
       if(aggr_too)
         DSC::dsc.aggregate(agreg_map, agg_level_name);
       else
         DSC::dsc.aggregate_replics(agreg_map, agg_level_name);
       T.stop();
     }
   }
   Notify_Time_Spend(T, "compilation of a SAN model\n");
 }
}



//====================================================================================//
//====================================================================================//
//====================================================================================//
//                               Compile_DESC Main                                    //
//====================================================================================//
int main(int argc, char *argv[])
{
 cout.setf(ios::scientific);
 cout.precision(16);
 set_new_handler(Communicate_Out_of_Memory);
 int compile_op = nothing; 
 
 Create_Temporary_Directories();
 strcpy(filename, ""); 

 if (argc >= 2){
  for (int i=1; i<argc; i++){
    if (argv[i][0] == '-'){
      if (!(strcmp(argv[i], "-std"))){
        if (compile_op == nothing)
          compile_op = compile;
         else
           cout << "warning: compile option is already set. Ignoring option \"-std\"." << endl;
      }
      if (!(strcmp(argv[i], "-aggr"))){
        if (compile_op == nothing)
          compile_op = comp_aggr;
        else
          cout << "warning: compile option is already set. Ignoring option \"-aggr\"." << endl;
      }
      if (!(strcmp(argv[i], "-lump"))){
        if (compile_op == nothing)
          compile_op = comp_lump;
        else
          cout << "warning: compile option is already set. Ignoring option \"-lump\"." << endl;
      }
      if ( !((!strcmp(argv[i], "-std")) || (!strcmp(argv[i], "-aggr")) || (!strcmp(argv[i], "-lump")) )){
        cout << "error: " << argv[i] << " invalid option" << endl; 
        exit (0);
      }
    }
    else{
      if (!strcmp(filename, ""))
        strcpy(filename, argv[i]);
      else
        cout << "warning: model name is already set as " << filename << ". Ignoring " << argv[i] << endl; 
    }
  }
 }
 
 switch (compile_op) {
//====================================================================================//
//                      Compiling and Normalizing Operations                          //
//====================================================================================//
   case nothing         : 
   case compile         : Compile_DSC(false, false); break;
   case comp_aggr       : Compile_DSC(true,  false); break;
   case comp_lump       : Compile_DSC(false, true ); break;

//====================================================================================//
//                                   Other Operations                                 //
//====================================================================================//
   case credits     : Show_Credits();                              break;
   default          : Programming_Error("invalid PEPS option", 0000)
 }
 Say_Goodbye();
 return(0);
}
